package com.persagy.apm.diagnose.timerecord.controller;

import com.persagy.apm.diagnose.timerecord.service.ICalculateTimeRecordService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.persagy.apm.diagnose.timerecord.model.*;
import com.persagy.apm.diagnose.timerecord.model.vo.*;
import com.persagy.apm.diagnose.timerecord.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.validation.Valid;
import java.util.List;

/**
 * 监测指标计算时间控制层
 * @author lixing
 * @version V1.0 2021-09-10 00:13:10
 */
@Api(tags = "监测指标计算时间") 
@Validated
@RestController
@RequestMapping("calculateTimeRecords")
public class CalculateTimeRecordController {
    @Autowired
    private ICalculateTimeRecordService calculateTimeRecordService;

	@ApiOperation(value = "创建监测指标计算时间")
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createCalculateTimeRecord(
	        @Valid @RequestBody AddCalculateTimeRecordDTO addCalculateTimeRecordDTO) {
        String id = calculateTimeRecordService.createCalculateTimeRecord(addCalculateTimeRecordDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "监测指标计算时间详情")
    @PostMapping("/get")
    public CommonResult<CalculateTimeRecordItemVO> queryCalculateTimeRecordDetail(
            @Valid @RequestBody QueryCalculateTimeRecordDetailDTO queryCalculateTimeRecordDetailDTO) {
        CalculateTimeRecord calculateTimeRecord = calculateTimeRecordService.queryCalculateTimeRecordDetail(queryCalculateTimeRecordDetailDTO.getId());
        CalculateTimeRecordItemVO calculateTimeRecordItemVO = ConvertCalculateTimeRecordTool.INSTANCE.convert2ResponseItemDTO(calculateTimeRecord);
        return ResultHelper.single(calculateTimeRecordItemVO);
    }


    @ApiOperation(value = "更新监测指标计算时间")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateCalculateTimeRecord(
            @Valid @RequestBody UpdateCalculateTimeRecordDTO updateCalculateTimeRecordDTO) {
        calculateTimeRecordService.updateCalculateTimeRecord(updateCalculateTimeRecordDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateCalculateTimeRecordDTO.getId()));
    }


    @ApiOperation(value = "删除监测指标计算时间")
    @PostMapping("/delete")
    public CommonResult deleteCalculateTimeRecord(
            @RequestBody @Valid DeleteCalculateTimeRecordDTO deleteCalculateTimeRecordDTO) {
        calculateTimeRecordService.deleteCalculateTimeRecord(deleteCalculateTimeRecordDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "监测指标计算时间列表")
    @PostMapping("/query")
    public CommonResult<PageList<CalculateTimeRecordListItemVO>> queryCalculateTimeRecord(
            @Valid @RequestBody QueryCalculateTimeRecordDTO queryCalculateTimeRecordDTO) {
        List<CalculateTimeRecord> calculateTimeRecordList = calculateTimeRecordService.queryCalculateTimeRecordList(queryCalculateTimeRecordDTO);
        List<CalculateTimeRecordListItemVO> resultList = ConvertCalculateTimeRecordTool.INSTANCE.convert2List(calculateTimeRecordList);
        return ResultHelper.multi(resultList);
    }
    
    //@ApiOperation(value = "监测指标计算时间列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<CalculateTimeRecordListItemVO>> queryCalculateTimeRecord(
    //        @Valid @RequestBody PageQueryCalculateTimeRecordDTO pageQueryCalculateTimeRecordDTO) {
    //    IPage<CalculateTimeRecord> pagedList = calculateTimeRecordService.pageQueryCalculateTimeRecord(pageQueryCalculateTimeRecordDTO);
    //    IPage<CalculateTimeRecordListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertCalculateTimeRecordTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，CalculateTimeRecord实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

