package com.persagy.apm.diagnose.utils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
public class RedisLock {

	@Autowired
	private RedisTemplate alarmRedisTemplate;

	/**
	 * 加锁
	 * 
	 * @param key
	 * @param value
	 *            当前时间+超时时间
	 * @return
	 */
	public boolean lock(String key, String value) {

		if (alarmRedisTemplate.opsForValue().setIfAbsent(key, value)) {
			// 相当于SETNX，setIfAbsent方法设置了为true,没有设置为false
			return true;
		}

		// 假设currentValue=A 接下来并发进来的两个线程的value都是B
		// 其中一个线程拿到锁,除非从始至终所有都是在并发（实际上这中情况是不存在的），只要开始时有数据有先后顺序，则分布式锁就不会出现“多卖”的现象
		String currentValue = String.valueOf(alarmRedisTemplate.opsForValue().get(key));
		// 如果锁过期 解决死锁
		if (!StringUtils.isEmpty(currentValue) && Long.parseLong(currentValue) < System.currentTimeMillis()) {
			// 获取上一个锁的时间，锁过期后，GETSET将原来的锁替换成新锁
			String oldValue = String.valueOf(alarmRedisTemplate.opsForValue().getAndSet(key, value));
			if (!StringUtils.isEmpty(oldValue) && oldValue.equals(currentValue)) {
				return true;
			}
		}
		// 拿到锁的就有执行权力，拿不到的只有重新再来
		return false;
	}

	/**
	 * 解锁
	 * 
	 * @param key
	 * @param value
	 */
	public void unlock(String key, String value) {
		try {
		    alarmRedisTemplate.delete(key);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    public void sendDataLock(String key, String value) {
//	    redisTemplate.opsForHash().put("ALARM-DATA-LOCK",key,value);
        alarmRedisTemplate.opsForValue().setIfAbsent(key, value,10,TimeUnit.SECONDS);
//        redisTemplate.expire(key,10, TimeUnit.SECONDS);
    }

}